###############################################################
#                    Call back functions                      #
###############################################################

proc not_implemented_yet {} {
  MessageDlg .msg -parent . -type ok -icon info -title "Hinweis" \
    -message "Diese Funktion ist zur Zeit nooch nicht implementiert!" 
}

proc progress_value { percent } {
  set GSM_Tk::prgindic $percent
}

proc status_message { msg } {
  global GSM_Tk::status
  set GSM_Tk::status $msg
}

proc error_message { msg } {
  MessageDlg .msg -parent . -type ok -icon error -justify left \
             -title "Fehlermeldung" -message $msg
}

proc update_record { } {
  global GSM_Tk::pbook_entries glb_act_pbook
  global entry_name entry_num entry_type
  global GSM_NUMBER_TYPE

  set cpos [.mainframe.sw.lbox curselection]
  if [string_empty $cpos] return

  set rec [lindex $GSM_Tk::pbook_entries($glb_act_pbook) $cpos]
  set diff1 [string_equal $entry_name [lindex $rec 1]]
  set diff2 [string_equal $entry_num  [lindex $rec 2]]
  if [expr $diff1 && $diff2] {return}
  
  if {[string index $entry_num 0] == "+"} {
    set typ $GSM_NUMBER_TYPE(international)
  } else {
    set typ $GSM_NUMBER_TYPE(unknown)
  }
  puts "DEBUG: [list [lindex $rec 0] $entry_name $entry_num $typ 1]"
  set GSM_Tk::pbook_entries($glb_act_pbook) [\
    lreplace $GSM_Tk::pbook_entries($glb_act_pbook) \
      $cpos $cpos [list [lindex $rec 0] $entry_name $entry_num $typ 1]]
  display_book $glb_act_pbook
  .mainframe.sw.lbox selection set $cpos
}

proc get_act_pbook { } {
  global gsm_tk_books

  set val [.mainframe.inp.cb getvalue]
  if {$val == -1} return ""
  set glb_act_pbook [lindex $gsm_tk_books $val]
  return $glb_act_pbook
}

proc read_all_pbooks { } {
  global gsm_tk_books GSM_TTY_DEVICE

  status_message "Einlesen der Telefonbucharten..."

  if {![info exists gsm_tk_books]} {
    set gsm_tk_books {}
    InitDevice $GSM_TTY_DEVICE
    foreach book [q_phone_books] {
      lappend gsm_tk_books [unquote $book]
      set GSM_Tk::pbook_used($book) 0
      set GSM_Tk::pbook_max($book) 0
    }
    CloseDevice
  }

  set rval {}
  foreach book $gsm_tk_books {
    lappend rval [lindex [get_gsm_pbook_desc $book] 0]
  }

  status_message ""
  return $rval
}

proc save_pbook { book } {
  global glb_pbk_prefix 
  global GSM_NUMBER_TYPE

  set pbk_file "${glb_pbk_prefix}_$book.tel"

  set ftypes {
    { "GSM_Tk Telefonbuch" ".tel" }
    { "Textdatei <TAB>"    ".txt" }
    { "CSV Datei>"         ".csv" }
  }
  puts "pbk_file=$pbk_file"
  set fil [tk_getSaveFile -defaultextension tel -parent . -filetypes $ftypes \
     -initialfile $pbk_file -title "Telefonbuch speichern"]
  if {$fil == ""} return

  set cnt 0
  set ext [file extension $fil]
  set fd [open $fil "w"] 
  puts "DEBUG: ext=$ext"
  foreach entr $GSM_Tk::pbook_entries($book) {
    set idx [lindex $entr 0]
    set nam [lindex $entr 1]
    set num [lindex $entr 2]
    if {[lindex $entr 3] == $GSM_NUMBER_TYPE(international)} {
      set typ 145 
    } else {
      set typ 129
    }
    switch $ext {
      .txt { set lin "$idx\t$nam\t$num\t$typ" }
      .csv { set lin "$idx,$nam,$num,$typ" }
      default { 
        set lin [list $book $idx $nam $num $typ]
      }
    }
    puts "DEBUG: $lin"
    puts $fd $lin
    incr cnt
    progress_value [expr int(100.0*$cnt/$GSM_Tk::pbook_used($book))] 
  }
  close $fd
  status_message "$cnt $book-Eintrge in Datei '[file tail $fil]' gespeichert"
  progress_value 0
}
######################################################
# 
# Menu callback functions
#
# naming convention: Callback[menu][submenu]
# e. g.: CallbackFileExit  
#
######################################################
proc CallbackFileAbout { } {
  global GSM_Tk::program GSM_Tk::version GSM_Tk::module 
  MessageDlg .msg -parent . -type ok -icon info -justify left \
            -title "Programminformation" -message \
"Programm: $GSM_Tk::program
Modul: $GSM_Tk::module
Version: $GSM_Tk::version
Autor: Gerhard Reithofer
(c) Techn. EDV Reithofer 2001-11"
}

proc CallbackFileExit { code } {
  destroy .
  exit $code
}

proc CallbackFileOpen   { } { 
  not_implemented_yet 
}

proc CallbackFileSave   { } { 

  set abk [get_act_pbook]

  if [string_empty $abk] {
    status_message "Fehler: Kein Telefonbuch ausgewhlt!"
    error_message "Es ist kein Telefonbuch aktiv!
Bitte zuerst ein Telefonbuch auswhlen."
    return
  }

  # if read_only...
  if [lindex [get_gsm_pbook_desc $abk] 1] { 
    status_message "Warnung: Dieses Telefonbuch kann nur gelesen werden!" 
    set res [MessageDlg .msg -parent . -icon warning -type okcancel \
              -width 500 -title "Telefonbuch kann nur gelesen werden" \
              -message "Dieses Telefonbuch kann nicht beschrieben werden!
Soll die Datei trotzdem gespeichert werden?"]
    if $res return 
  }

  save_pbook $abk
}

proc CallbackDataRead { reread } {
  global GSM_TTY_DEVICE glb_act_pbook
  global GSM_Tk::pbook_entries GSM_Tk::pbook_max GSM_Tk::pbook_used
 
  set abk [get_act_pbook]
  if [string_empty $abk] {
    status_message "Fehler: Kein Telefonbuch ausgewhlt!"
    error_message "Es ist kein Telefonbuch aktiv!
Bitte zuerst ein Telefonbuch auswhlen."
    return
  }

  set glb_act_pbook $abk

  if [lindex [get_gsm_pbook_desc $glb_act_pbook] 1] { ; ### readonly?
    set def_bg [.mainframe.inp.ty cget -entrybg]
    .mainframe setmenustate write_to disabled
    .mainframe.inp.na configure -editable 0
    .mainframe.inp.nu configure -editable 0
    .mainframe.inp.nu configure -entrybg $def_bg
    .mainframe.inp.na configure -entrybg $def_bg
  } else {
    .mainframe setmenustate write_to active
    .mainframe.inp.na configure -editable 1
    .mainframe.inp.nu configure -editable 1
    .mainframe.inp.nu configure -entrybg white
    .mainframe.inp.na configure -entrybg white
  }

  if [info exists GSM_Tk::pbook_entries($glb_act_pbook)] {
    if {!$reread} { 
      display_book $glb_act_pbook
      return 
    }
  }
  
  InitDevice $GSM_TTY_DEVICE
  set res [s_phone_book $glb_act_pbook]
  set caps [q_caps_act_pbook]

  set GSM_Tk::pbook_used($glb_act_pbook) [lindex $caps 1]
  set GSM_Tk::pbook_max($glb_act_pbook) [lindex $caps 2]
 
  set conts [q_pbook_conts $glb_act_pbook]
  set range [split [lindex $conts 0] "-"]
  set min [lindex $range 0]
  set max [lindex $range 1]
  set lnu [lindex $conts 1]
  set lna [lindex $conts 2]

  set GSM_Tk::pbook_entries($glb_act_pbook) {}
  .mainframe.sw.lbox delete 0 end 
  
  status_message "Telefonbuch $glb_act_pbook wird gelesen..."
  set rval {}
  for {set i $min} {$i <= $GSM_Tk::pbook_used($glb_act_pbook)} {incr i} {
    set entry [r_pbook_entry $i]
    set idx                   [lindex $entry 0]
    set num [unquote          [lindex $entry 1]]
    set cod [gsm_number_type  [lindex $entry 2]]
    set nam [unquote [pdu2iso [lindex $entry 3]]]
    set entr [list $idx $nam $num $cod 0]
    lappend GSM_Tk::pbook_entries($glb_act_pbook) $entr
    progress_value [expr int(100.0*$i/$GSM_Tk::pbook_used($glb_act_pbook))]
  }

  display_book $glb_act_pbook
  status_message ""

  CloseDevice

  progress_value 0
}

proc CallbackDataWrite { } { not_implemented_yet }

proc CallbackEditCreate    { } { not_implemented_yet }
proc CallbackEditChange    { } { not_implemented_yet }
proc CallbackEditDelete    { } { not_implemented_yet }
proc CallbackEditDeleteAll { } { not_implemented_yet }
proc CallbackEditSearch    { } { not_implemented_yet }
