#####################################################################
#
#     TCL Siemens library 0.2
#    by Gerhard Reithofer
# mail: gerhardr@tech-edv.co.at
#
# Techn. EDV Reithofer, Technical software solutions
# http://www.tech-edv.co.at
#
#
# WARNING - WARNING - WARNING - WARNING - WARNING - WARNING - WARNING
# Many of these functions are untested!
# Yor will loose information if working with this software components!
# DISCLAIMER
# In no case I will be liable for any loss of information or
# any other misfortune, calamity, disaster ... which may be caused
# using this software or components of it!
# WARNING - WARNING - WARNING - WARNING - WARNING - WARNING - WARNING

set debuglevel 0
set GSM_TTY_DEVICE /dev/modem
 
#
# end of user modifyable settings
#

set SIO_SPEED 19200
set STTY_PROG /bin/stty

set GSM_DEVICE(dev) ""
set GSM_DEVICE(buffer) {}
set GSM_DEVICE(data) 0

set GSM_PHONE_BOOKS [list SM FD DC MD ON OW LD ME MC MS RC CD]

set GSM_PHONE_BOOK(FD) "fix gespeichert"
set GSM_PHONE_BOOK(SM) "SIM Karte"
set GSM_PHONE_BOOK(DC) "zuletzt gewhlt, Handy intern"
set GSM_PHONE_BOOK(MD) $GSM_PHONE_BOOK(DC)
set GSM_PHONE_BOOK(ON) "eigene Nummern"
set GSM_PHONE_BOOK(OW) $GSM_PHONE_BOOK(ON)
set GSM_PHONE_BOOK(LD) "zuletzt gewhlt, SIM Karte"
set GSM_PHONE_BOOK(ME) "Handy intern"
set GSM_PHONE_BOOK(MC) "nicht angenommene Anrufe"
set GSM_PHONE_BOOK(MS) $GSM_PHONE_BOOK(MC)
set GSM_PHONE_BOOK(RC) "angenommene Anrufe"
set GSM_PHONE_BOOK(CD) $GSM_PHONE_BOOK(RC)
set GSM_PHONE_BOOK(unknown) "Unbekannter Eintrag:"

set GSM_WRITABLE [list SM FD ON OW ME] 

set GSM_NUMBER_TYPE(international) "international"
set GSM_NUMBER_TYPE(unknown)       "unbekannt"
set GSM_NUMBER_TYPE(reserved)      "* Fehler *"

proc gsm_number_type { num } {
  global GSM_NUMBER_TYPE
  switch $num {
    129     { set rval $GSM_NUMBER_TYPE(unknown)       }
    145     { set rval $GSM_NUMBER_TYPE(international) }
    default { set rval $GSM_NUMBER_TYPE(reserved)      }
  }
  return $rval
}

proc get_gsm_pbook_desc { pbook } {
  global GSM_WRITABLE GSM_PHONE_BOOKS GSM_PHONE_BOOK
  set pbk [unquote $pbook] ; ### just for sure...
	set readonly [expr [lsearch $GSM_WRITABLE $pbk] == -1]
  if {[lsearch $GSM_PHONE_BOOKS $pbk] == -1} {
	  set rval "$GSM_PHONE_BOOK(unknown) $pbk"
  } else {
	  set rval $GSM_PHONE_BOOK($pbk)
	}
	return [list $rval $readonly];
}

################################################################
#
# Debugging function
#
proc DebugOut {level msg} {
  global debuglevel
  if {$level < $debuglevel} { puts "DEBUG: $msg" }
}

#
# If format is "from-to" we extract it to "from,from+1, ... ,to-1,to"
#
proc expand { region } {
  DebugOut 3 "CALL: expand $region"
  if { [string first "-" $region] >= 0 } {
    set se [split $region "-"]
    set i1 [lindex $se 0]
    set i2 [lindex $se 1]
    set rval "$i1"
    while {[expr [incr i1] <= $i2]} { append rval ",$i1" }
  } else {
    set rval $region
  }
  DebugOut 3 "RETURN: expand $rval"
  return $rval
}

###################################################
#
# Remove the annoying quotes around an entry
#
proc unquote { str } {
  set idx [expr [string length $str]-1]
  if {[string index $str $idx] == "\""} {
    set str [string range $str 0 [expr $idx-1]]
  }
  if {[string index $str 0] == "\""} {
    set str [string range $str 1 end]
  }
  return $str
}

############################################
#
# low level I/O
#
############################################
#
# GSM_DEVICE(dev) file event callback
#  this is the basic device reader event handler
#
proc GSM_Reader {} {
  DebugOut 9 "CALL GSM_Reader"
  global GSM_DEVICE
  gets $GSM_DEVICE(dev) line
  lappend GSM_DEVICE(buffer) $line
  incr GSM_DEVICE(data)
}

##############################################
# open the GSM_DEVICE file and set callbacks 
#
# at the moment I am using the unportable 'stty initialisation', 
# because it works well under unix and my TCL/TK version has a 
# broken fconfigure function
#
# Information found in TkUsr, a great program that allows the
#  playing of messages and retreiving of faxes stored on
#  USRobotics / 3Com message modems.
# by Ludovic Drolez (ldrolez@usa.net)
#          http://ldrolez.free.fr/tkusr/
#
# His version looks like:
#    set m "115200,n,8,1"
#    set device(dev) [open $dev {RDWR}]
#    fconfigure $device(dev) -mode $m -buffering none -translation auto \
#                            -blocking $device(blocking)
#    fileevent $device(dev) readable {GSM_Reader}
#    set device(data) 0
#    set test [fconfigure $device(dev) -mode]
#    if {$test != $m} {
#  tk_messageBox -icon error -title "Buggy Tcl" -type ok -message "
# The serial test returned '$test'.
# ... etc, etc ...
#
proc InitDevice { dev } {
  global GSM_DEVICE STTY_PROG SIO_SPEED
  DebugOut 9 "CALL InitDevice $dev"
  set GSM_DEVICE(dev) [open $dev {RDWR}]
  DebugOut 8 "GSM_DEVICE(dev)=$GSM_DEVICE(dev)"
  exec $STTY_PROG -F $dev -brkint time 5 ispeed $SIO_SPEED ospeed $SIO_SPEED \
       ixoff -opost isig -ignbrk -ignpar -icrnl -imaxbel -onlcr -icanon \
       -iexten -echoke -echo -echoe -echok -echoctl
  fconfigure $GSM_DEVICE(dev) -translation {binary crlf} \
                              -buffering none -blocking 1
  fileevent $GSM_DEVICE(dev) readable {GSM_Reader}
  set GSM_DEVICE(data) 0
}

#
# get data until the next $pattern or 'ERROR'
#
proc ReadUntil { pattern } {
  global GSM_DEVICE
  DebugOut 3 "CALL ReadUntil $pattern"
 
  set last ""
  set rval ""
  set result {}
  while {[string first $pattern $last]&&
         [string first "ERROR" $last] == -1} {
    if {[llength $GSM_DEVICE(buffer)] == 0} {
      tkwait variable GSM_DEVICE(data)
      set GSM_DEVICE(data) 0
    }
    set last [string trim [lindex $GSM_DEVICE(buffer) 0] "\x0d\x0a"]
    set GSM_DEVICE(buffer) [lrange $GSM_DEVICE(buffer) 1 1000]
    if {[string compare $last ""] && [string compare $last $pattern]} {
      lappend result $last
    }
  }
  DebugOut 3 "RETURN: ReadUntil $result"
  return $result
}

#
# send a simple string to the modem and wait for an OK-response 
#
proc ModemCommand { str } {
  global GSM_DEVICE
  DebugOut 9 "CALL ModemCommand '$str'"
  puts $GSM_DEVICE(dev) $str
  after 100
  set result [ ReadUntil "OK" ]
  DebugOut 9 "RETURN: ModemCommand $result"
  return $result
}

#
# Self explanatory - I hope
#
proc CloseDevice {} {
  global GSM_DEVICE
  DebugOut 9 "CALL CloseDevice"
  close $GSM_DEVICE(dev)
}

############################################
#
# list formating routines
#
############################################
#
# return of a token encoded string
#
# deals with strings like:
#     "+CPBS: a,b,c"
# whereas "+CPBS:" is the requested token in $alist
#
# it returns the string excluding the token prefix at 
# or "ERROR:" and the input $alist
#
proc get_result_list {code alist} {
  set clen [string length $code]
  set pref [string range $alist 0 [expr $clen-1]]
  set rslt [string trim [string range $alist $clen end]]
  if [string compare $pref $code] {
    return [list ERROR: $alist]
  } else {
    return $rslt
  }
}

############################################################
#
# return list of "(" ")" delimited, "," separated entries
# "," will be translated to list separators,
# "(" and ")" will be translated to sublists
# e.g.:
# (2,"D2 PRIVAT",,26202),(3,"D1-TELEKOM",,26201),,(0,1,3,4) ,(0,2)
# will be translated to the tcl list:
# {2 "D2 PRIVAT" "" 26202 } {3 "D1-TELEKOM" "" 26201} "" {0 1 3 4} {0 2}
#
# was a tricky thing - I hope it works now
#
proc get_separated_list { str } {
DebugOut 3 "CALL: get_separated_list '$str'"
  set i 0
  set rval {}
  set entr ""
  set sl [string length $str]
  while {$i<$sl} {
    set c1 [string index $str $i]
    if {$c1 == ","} {
      lappend rval $entr
      set entr ""
    } elseif {$c1 == "("} {
### search matching closing parenthis - begin
      set rest [string range $str [expr $i+1] end]
      set rl [string length $rest]
      set level 1
      set closed 0
      while {$level} {
        set c2 [string index $rest $closed]
        if {$c2 == "("} {
          incr level
        } elseif {$c2 == ")"} {
          incr level -1
        }
        if {[incr closed]>=$rl} {set $level 0}
      }
### search matching closing parenthis - end
      set from 0
      set to [expr $closed-2]
      set tmp [string range $rest $from $to]
      set rlist [get_separated_list $tmp]
      lappend rval $rlist
      set i [expr $i+$closed]
      # if a separator follows et end of the
      # list we skip it
      if {[string index $str [expr $i+1]]==","} {incr i}
      set entr ""
    } else {
      append entr $c1
    }
    incr i
  }
  if [string compare "" $entr] {lappend rval $entr}
DebugOut 3 "RETURN: get_separated_list $rval"
  return $rval
}

############################################
#
# Modem specific commands - Hayes compatible
#
############################################
#
# dial voice number
# param.: number
# result: modem result (OK/ERROR)
#  number may be a valid pnone number (incl. "+"), "L" (for last dialed)
#  or "S(x)" (x: 1..9 for recentley dialed number1..9)
proc dial_voice_number { number } {
  set at_cmd "ATD"
  append at_cmd $number
  set mdmres [ModemCommand $at_cmd]
  return [lindex $mdmres 1]
}

#
# hangup line
# param.: none
# result: modem result (OK/ERROR)
proc hangup_line { } {
  set at_cmd "AT+CHUP"
  set mdmres [ModemCommand $at_cmd]
  return [lindex $mdmres 1]
}


#################################################################
#
# Graphic/Tone/Calendar entries 
#     Is that Siemens-only specific???
#
#################################################################
#
# Naming conventions - not fully consistent 
#
# q_* ... query status 
# r_* ... read values
# s_* ... set, select or write values
# 
# query graphic, tone and calendar entries
# param.: none
# result: {typ range} ... e.g. {"vcs"} 1-30
# known types: bmp (bit map), mid (midi sound), vcs (vvalendar)
# ranges: single number or $from-$to
# example: {"bmp" 0-1} {"mid" 0} {"vcs" 1-30}
#
proc q_gtc_entries {} {
  set at_cmd "AT^SBNR=?"
  set mdmres [ModemCommand $at_cmd]
  set caps [get_result_list "^SBNR:" [lindex $mdmres 1]]
  set rval [get_separated_list $caps]
  return $rval
} 

#
# read graphic, tone and calendar entries
# param.: typ (bmp, mid or vcs)               (see q_gtc_entries)
#         idx - index of entry in valid range (see q_gtc_entries)
# result: type,subtype,1/2,numpakets,hexdata
#  known types:
#    bmp - hex codes of bitmaps
#      known subtypes:
#        0 - shown permanentely
#        1 - shown temporarily (only s35)
#    mid - hex codes of midi sound
#      known subtypes:
#        0 - 1st and only entry
#    vcs - hex codes of vcal entry - format: http://www.imc.org/pdi
#      known subtypes:
#        0 - entry 1
#        1 - entry 2
#   etc. - until max. $idx
#
# max. size for hexdata is: 176 bytes
#
# vcal example:
#   BEGIN:VCALENDAR
#   VERSION:1.0
#   BEGIN:VEVENT
#   CATEGORIES:MEETING
#   DTSTART:20011005T122000
#   RRULE:MD1 05
#   DESCRIPTION:Abendessen mit C.
#   END:VEVENT
#   END:VCALENDAR
#
# known defintions: (according s25obex.pdf)
#   AALARM:JJJJMMSSTHHMMSS - audio alarm e.g.: AALARM:20011005T122000
#   CATEGORIES:
#     ANNIVERSARY  - birthday/Geburtstag
#     MEETING      - meeting /Treffen
#     MICELLANEOUS - memo    /Memo
#     PHONE CALL   - call    /Anruf - phone number in DESCRIPTION field
#   DALARM:JJJJMMSSTHHMMSS - display alarm (deactiveted if not set)
#   DESCRIPTION - description text (max. 16 chars.)
#   DSTART:JJJJMMSSTHHMMSS - start of event
#   RRULE:$RULE - event reoccurence
#     known $RULEs: D1 - daily
#                   W1 - weekly
#                   D7 - weekly
#                   Y1 - yearly
# 
proc r_gtc_entry { typ idx } {
  set at_cmd "AT^SBNR="
  append at_cmd "$typ,$idx"
  set mdmres [ModemCommand $at_cmd]
  set rlist [get_result_list "^SBNR:" "[lindex $mdmres 1],[lindex $mdmres 2]"]
  set caps [get_result_list "^SBNR:" [lindex $mdmres 1]]
  set rval [get_separated_list $rlist]
  return $rval
}

#
# write graphic, tone and calendar entries
# param.: typ (bmp, mid or vcs) - see q_gtc_entries
#         idx - index of entry in valid range (see q_gtc_entries)
#         act - actual paket number
#         max - max. number of paket
#         dat - binary data as 8 bit string
# result: OK/ERROR
#
# remark: newline characters must be converted to "0D0A"
# 
proc s_gtc_entry { typ idx act max dat } {
  set at_cmd "AT^SBNW="
  set dlen [string length $dat]
  append at_cmd "$typ,$idx,$max\r\n"
  for {set i 0} {i<dlen} {incr i} {
    scan [string index $dat $i] "%c" bin
    append at_cmd [format "%02X" bin]
  }
  append at_cmd [format "%c" 0x1a]
  set mdmres [ModemCommand $at_cmd]
  return [lindex $mdmres 1]
}

#
# There are many others, which I never used 
# Information found at Siemens's
#
#     Manual Reference
#      AT Command Set 
#    (GSM 07.07, GSM 07.05)
#  for SIEMENS Mobile Phone
#          S25
#    and Derivatives   
# found at
#  http://www.min.at/prinz/s25/s25at.pdf
# the real great
#      (M)obile (I)nformation (N)etwork
# site by Richard Prinz Richard.Prinz@min.at 
#
